import { BeEvent, ClientRequestContext } from "@bentley/bentleyjs-core";
import { FrontendAuthorizationClient } from "@bentley/frontend-authorization-client";
import { AccessToken, IncludePrefix, UserInfo } from "@bentley/itwin-client";
/**
 * Implements AccessToken that uses Basic access authentication
 * @internal
 */
export declare class BasicAccessToken extends AccessToken {
    constructor(tokenStr?: string);
    /**
     * Create a BasicAccessToken from user credentials
     * @param userCredentials User credentials containing email and password of the user.
     */
    static fromCredentials(userCredentials: any): AccessToken;
    /**
     * Creates a token to be used in Authorization header.
     * @param includePrefix Set to Yes if prefix (Basic) should be included before the token.
     */
    toTokenString(includePrefix?: IncludePrefix): string;
    /**
     * initialize the tokenString field of the current instance of BasicAccessToken
     * @param tokenStr String representation of the token
     */
    initFromTokenString(tokenStr: string): void;
}
/** Implements the user permission abstraction by creating a BasicAccessToken. Note that the corresponding IModelBank server must
 * be able to tolerate this BasicAccessToken.
 * @internal
 */
export declare class IModelBankBasicAuthorizationClient implements FrontendAuthorizationClient {
    private _userCredentials;
    private _token?;
    constructor(_userInfo: UserInfo | undefined, _userCredentials: any);
    signIn(_requestContext?: ClientRequestContext): Promise<void>;
    signOut(_requestContext?: ClientRequestContext): Promise<void>;
    readonly onUserStateChanged: BeEvent<(token: AccessToken | undefined) => void>;
    get isAuthorized(): boolean;
    get hasExpired(): boolean;
    get hasSignedIn(): boolean;
    getAccessToken(_requestContext?: ClientRequestContext): Promise<AccessToken>;
}
//# sourceMappingURL=IModelBankBasicAuthorizationClient.d.ts.map