"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var BasicAccessToken_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelBankBasicAuthorizationClient = exports.BasicAccessToken = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const loggerCategory = itwin_client_1.ITwinClientLoggerCategory.Authorization;
/**
 * Implements AccessToken that uses Basic access authentication
 * @internal
 */
let BasicAccessToken = BasicAccessToken_1 = class BasicAccessToken extends itwin_client_1.AccessToken {
    constructor(tokenStr) {
        super(tokenStr, undefined, undefined, undefined);
        this.setPrefix("Basic");
    }
    /**
     * Create a BasicAccessToken from user credentials
     * @param userCredentials User credentials containing email and password of the user.
     */
    static fromCredentials(userCredentials) {
        const basicToken = new BasicAccessToken_1("");
        basicToken._tokenString = Buffer.from(`${userCredentials.email}:${userCredentials.password}`).toString("base64");
        return basicToken;
    }
    /**
     * Creates a token to be used in Authorization header.
     * @param includePrefix Set to Yes if prefix (Basic) should be included before the token.
     */
    toTokenString(includePrefix = itwin_client_1.IncludePrefix.Yes) {
        let token = "";
        if (includePrefix === itwin_client_1.IncludePrefix.Yes)
            token += `${this._prefix} `;
        token += this._tokenString;
        return token;
    }
    /**
     * initialize the tokenString field of the current instance of BasicAccessToken
     * @param tokenStr String representation of the token
     */
    initFromTokenString(tokenStr) {
        if (!tokenStr.startsWith(this._prefix)) {
            throw new bentleyjs_core_1.BentleyError(bentleyjs_core_1.AuthStatus.Error, "Invalid access token", bentleyjs_core_1.Logger.logError, loggerCategory, () => ({ tokenStr }));
        }
        const userPass = tokenStr.substr(this._prefix.length + 1);
        this._tokenString = userPass;
    }
};
BasicAccessToken = BasicAccessToken_1 = __decorate([
    itwin_client_1.TokenPrefix("Basic")
], BasicAccessToken);
exports.BasicAccessToken = BasicAccessToken;
/** Implements the user permission abstraction by creating a BasicAccessToken. Note that the corresponding IModelBank server must
 * be able to tolerate this BasicAccessToken.
 * @internal
 */
class IModelBankBasicAuthorizationClient {
    constructor(_userInfo, _userCredentials) {
        this._userCredentials = _userCredentials;
        this.onUserStateChanged = new bentleyjs_core_1.BeEvent();
    }
    async signIn(_requestContext) {
        _requestContext === null || _requestContext === void 0 ? void 0 : _requestContext.enter();
        this._token = BasicAccessToken.fromCredentials(this._userCredentials);
        this.onUserStateChanged.raiseEvent(this._token);
    }
    async signOut(_requestContext) {
        _requestContext === null || _requestContext === void 0 ? void 0 : _requestContext.enter();
        this._token = undefined;
        this.onUserStateChanged.raiseEvent(this._token);
    }
    get isAuthorized() {
        return !!this._token;
    }
    get hasExpired() {
        return !this._token;
    }
    get hasSignedIn() {
        return !!this._token;
    }
    async getAccessToken(_requestContext) {
        if (!this._token) {
            throw new Error("User is not signed in.");
        }
        return this._token;
    }
}
exports.IModelBankBasicAuthorizationClient = IModelBankBasicAuthorizationClient;
//# sourceMappingURL=IModelBankBasicAuthorizationClient.js.map