"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelBankDummyAuthorizationClient = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
class DummyAccessToken extends itwin_client_1.AccessToken {
    /** Sets up a new AccessToken based on some generic token abstraction used for iModelBank use cases
     * @internal
     */
    static fromForeignProjectAccessTokenJson(foreignJsonStr) {
        if (!foreignJsonStr.startsWith(`{\"${this.foreignProjectAccessTokenJsonProperty}\":`))
            return undefined;
        const props = JSON.parse(foreignJsonStr);
        if (props[this.foreignProjectAccessTokenJsonProperty] === undefined)
            return undefined;
        const tok = new DummyAccessToken(foreignJsonStr);
        const userInfoJson = props[this.foreignProjectAccessTokenJsonProperty].userInfo;
        const userInfo = itwin_client_1.UserInfo.fromJson(userInfoJson);
        tok.setUserInfo(userInfo);
        return tok;
    }
}
DummyAccessToken.foreignProjectAccessTokenJsonProperty = "ForeignProjectAccessToken";
/** Implements the user permission abstraction by creating a dummy AccessToken. Note that the corresponding IModelBank server must
 * be able to tolerate this dummy token.
 * @internal
 */
class IModelBankDummyAuthorizationClient {
    constructor(_userInfo, _userCredentials) {
        this._userInfo = _userInfo;
        this._userCredentials = _userCredentials;
        this.onUserStateChanged = new bentleyjs_core_1.BeEvent();
    }
    async signIn() {
        if (!this._userInfo) {
            this._userInfo = {
                id: "",
                email: {
                    id: this._userCredentials.email,
                },
                profile: {
                    name: "",
                    firstName: "",
                    lastName: "",
                },
            };
        }
        const foreignAccessTokenWrapper = {};
        foreignAccessTokenWrapper[DummyAccessToken.foreignProjectAccessTokenJsonProperty] = { userInfo: this._userInfo };
        this._token = DummyAccessToken.fromForeignProjectAccessTokenJson(JSON.stringify(foreignAccessTokenWrapper));
        this.onUserStateChanged.raiseEvent(this._token);
    }
    async signOut() {
        this._token = undefined;
        this.onUserStateChanged.raiseEvent(this._token);
    }
    get isAuthorized() {
        return !!this._token;
    }
    get hasExpired() {
        return !this._token;
    }
    get hasSignedIn() {
        return !!this._token;
    }
    async getAccessToken(_requestContext) {
        if (!this._token) {
            throw new Error("User is not signed in.");
        }
        return this._token;
    }
}
exports.IModelBankDummyAuthorizationClient = IModelBankDummyAuthorizationClient;
//# sourceMappingURL=IModelBankDummyAuthorizationClient.js.map