"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelBankHandler = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelBankClient
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const BaseHandler_1 = require("../imodelhub/BaseHandler");
/**
 * This class acts as the WsgClient for other iModelBank Handlers.
 * @internal
 */
class IModelBankHandler extends BaseHandler_1.IModelBaseHandler {
    /**
     * Creates an instance of IModelBankWsgClient.
     * @param handler The upload/download handler to use -- backends only.
     * @param keepAliveDuration TBD
     */
    constructor(url, handler, keepAliveDuration = 30000) {
        super(keepAliveDuration, handler);
        this.baseUrl = url;
        if (url.startsWith("http://")) {
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            this._agent = require("http").Agent({ keepAlive: keepAliveDuration > 0, keepAliveMsecs: keepAliveDuration });
        }
    }
    getUrlSearchKey() { bentleyjs_core_1.assert(false, "Bentley cloud-specific method should be factored out of WsgClient base class"); return ""; }
    async getUrl(_requestContext, excludeApiVersion) {
        if (this._url)
            return this._url;
        this._url = this.baseUrl;
        if (!excludeApiVersion) {
            this._url += `/${this.apiVersion}`;
        }
        return this._url;
    }
}
exports.IModelBankHandler = IModelBankHandler;
//# sourceMappingURL=IModelBankHandler.js.map