"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BriefcaseHandler = exports.BriefcaseQuery = exports.Briefcase = exports.BriefcaseAccessMode = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const Locks_1 = require("./Locks");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/** Controls whether the user has exclusive or shared access to a local briefcase
 * @internal
 */
var BriefcaseAccessMode;
(function (BriefcaseAccessMode) {
    BriefcaseAccessMode[BriefcaseAccessMode["Shared"] = 0] = "Shared";
    BriefcaseAccessMode[BriefcaseAccessMode["Exclusive"] = 1] = "Exclusive";
})(BriefcaseAccessMode = exports.BriefcaseAccessMode || (exports.BriefcaseAccessMode = {}));
/**
 * Briefcase is a copy of the master file, that user acquires to work with the iModel. Briefcase instance represents metadata about a copy of iModel's master file.
 *
 * File properties describe the file that would be downloaded through downloadUrl. It is the most recently updated copy of master file that is stored on iModelHub. These copies do not necessarily have the latest [[ChangeSet]] applied to them.
 *
 * briefcaseId is the id that user needs to write into the local copy of master file and use for other iModelHub requests. briefcaseId ranges from 2 to 16777215, see [BriefcaseId]($backend).
 * @internal
 */
let Briefcase = class Briefcase extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileName")
], Briefcase.prototype, "fileName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileDescription")
], Briefcase.prototype, "fileDescription", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileSize")
], Briefcase.prototype, "fileSize", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileId")
], Briefcase.prototype, "fileId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.BriefcaseId"),
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "briefcaseId")
], Briefcase.prototype, "briefcaseId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.UserOwned"),
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "userId")
], Briefcase.prototype, "userId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.MergedChangeSetId")
], Briefcase.prototype, "mergedChangeSetId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.AcquiredDate")
], Briefcase.prototype, "acquiredDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ExpirationDate")
], Briefcase.prototype, "expirationDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.DeviceName")
], Briefcase.prototype, "deviceName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ChangeSetIdOnDevice")
], Briefcase.prototype, "changeSetIdOnDevice", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.IsReadOnly"),
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "isReadOnly")
], Briefcase.prototype, "isReadOnly", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[FileAccessKey].relatedInstance[AccessKey].properties.DownloadUrl")
], Briefcase.prototype, "downloadUrl", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[CreatedByApplication].relatedInstance[Application].properties.Id")
], Briefcase.prototype, "applicationId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[CreatedByApplication].relatedInstance[Application].properties.Name")
], Briefcase.prototype, "applicationName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "accessMode")
], Briefcase.prototype, "accessMode", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "localPathname")
], Briefcase.prototype, "localPathname", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "lastAccessedAt")
], Briefcase.prototype, "lastAccessedAt", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("ecdb", "iModelId")
], Briefcase.prototype, "iModelId", void 0);
Briefcase = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.Briefcase", { schemaPropertyName: "schemaName", classPropertyName: "className" }),
    itwin_client_1.ECJsonTypeMap.classToJson("ecdb", "ServiceStore.Briefcase", { classKeyPropertyName: "className" })
], Briefcase);
exports.Briefcase = Briefcase;
/**
 * Query object for getting [[Briefcase]]s. You can use this to modify the [[BriefcaseHandler.get]] results.
 * @internal
 */
class BriefcaseQuery extends itwin_client_1.WsgQuery {
    /**
     * Query single [[Briefcase]] by its id. If briefcase is not found, request will be rejected with a [[WsgError]] and status [WSStatus.InstanceNotFound]($bentley).
     * @param id Id of the Briefcase.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if id is undefined or it is not a valid Briefcase id value.
     */
    byId(id) {
        Errors_1.ArgumentCheck.validBriefcaseId("id", id);
        this._byId = id;
        return this;
    }
    /**
     * Used by handler to get the id that is queried.
     * @returns Value that was set with byId method.
     * @internal
     */
    getId() {
        return this._byId;
    }
    /**
     * Query will additionally select [[Briefcase]] file download URL. This is needed to use the Briefcase object with [[BriefcaseHandler.download]].
     * @returns This query.
     */
    selectDownloadUrl() {
        HubQuery_1.addSelectFileAccessKey(this._query);
        return this;
    }
    /**
     * Query will additionally select data about application that created this [[Briefcase]].
     * @returns This query.
     */
    selectApplicationData() {
        HubQuery_1.addSelectApplicationData(this._query);
        return this;
    }
    /**
     * Query will select [[Briefcase]]s owned by this user.
     * @returns This query.
     */
    ownedByMe() {
        this.addFilter("UserOwned+eq+'@me'");
        return this;
    }
}
exports.BriefcaseQuery = BriefcaseQuery;
/**
 * Handler for managing [[Briefcase]]s. Use [[IModelClient.Briefcases]] to get an instance of this class.
 * In most cases, you should use [BriefcaseDb]($backend) methods instead.
 * @internal
 */
class BriefcaseHandler {
    /** Constructor for BriefcaseHandler. Use [[IModelClient]] instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @param fileHandler Handler for file system.
     * @internal
     */
    constructor(handler, imodelClient, fileHandler) {
        this._handler = handler;
        this._imodelClient = imodelClient;
        this._fileHandler = fileHandler;
    }
    /** Get relative url for Briefcase requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcaseId Id of the briefcase.
     */
    getRelativeUrl(iModelId, briefcaseId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/Briefcase/${briefcaseId || ""}`;
    }
    /** Acquire a [[Briefcase]] for the specified iModel. This assigns you a new briefcaseId and returns you a download link.
     * A briefcase is automatically acquired when calling [BriefcaseManager.download]($backend) or [BriefcaseDb.create]($backend). You should use this method only when you want to acquire the briefcaseId without downloading the file. If you need just the download link, you can call [[BriefcaseHandler.get]] with [[BriefcaseQuery.selectDownloadUrl]].
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcase Information of the Briefcase to acquire.
     * @returns The acquired Briefcase instance.
     * @throws [[IModelHubError]] with [IModelHubStatus.MaximumNumberOfBriefcasesPerUser]($bentley) or [IModelHubStatus.MaximumNumberOfBriefcasesPerUserPerMinute]($bentley) if a limit of Briefcases for that user was reached. Users should use the Briefcases they have previously acquired. If that is no longer possible, they should delete them, to be able to acquire new ones.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async create(requestContext, iModelId, briefcase = new Briefcase()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Acquiring briefcase for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        briefcase = await this._handler.postInstance(requestContext, Briefcase, this.getRelativeUrl(iModelId), briefcase);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Acquired briefcase for iModel", () => briefcase);
        return briefcase;
    }
    /** Update the [[Briefcase]] of an iModel. Only [[Briefcase.deviceName]] and [[Briefcase.changeSetIdOnDevice]] can be changed when updating the briefcase.
     * Briefcase expiration date is also extended with each update request for a configured period of time (default is 30 days).
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcase Briefcase to update.
     * @returns Updated Briefcase instance from iModelHub.
     * @throws [[IModelHubError]] with [IModelHubStatus.FailedToGetProductSettings]($bentley) if request to get settings to Product Settings service fails.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, iModelId, briefcase) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Updating briefcase for iModel", () => ({ iModelId, briefcase }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const briefcaseId = parseInt(briefcase.wsgId, 10);
        Errors_1.ArgumentCheck.validBriefcaseId("briefcase.wsgId", briefcaseId);
        const updatedBriefcase = await this._handler.postInstance(requestContext, Briefcase, this.getRelativeUrl(iModelId, briefcaseId), briefcase);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Updated briefcase for iModel", () => ({ iModelId, briefcase }));
        return updatedBriefcase;
    }
    /** Delete the [[Briefcase]] from iModelHub. This frees up the id to be reused later and allows user to acquire additional briefcases if one of the briefcase limits was reached.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcaseId Id of the Briefcase to be deleted.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async delete(requestContext, iModelId, briefcaseId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Deleting briefcase from iModel", () => ({ iModelId, briefcaseId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        const existingLocksOwnedByBriefcase = await this._imodelClient.locks.get(requestContext, iModelId, new Locks_1.LockQuery().byBriefcaseId(briefcaseId).top(1));
        requestContext.enter();
        if (existingLocksOwnedByBriefcase.length > 0) {
            try {
                await this._imodelClient.locks.deleteAll(requestContext, iModelId, briefcaseId);
                requestContext.enter();
            }
            catch (error) {
                requestContext.enter();
                if (!(error instanceof Errors_1.IModelHubError) || error.errorNumber !== bentleyjs_core_1.IModelHubStatus.UserDoesNotHavePermission) {
                    throw error;
                }
            }
        }
        await this._handler.delete(requestContext, this.getRelativeUrl(iModelId, briefcaseId));
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Deleted briefcase from iModel", () => ({ iModelId, briefcaseId }));
    }
    /** Get the [[Briefcase]]s.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried Briefcases or select different data from them.
     * @returns Briefcases that match the query.
     * @throws [[WsgError]] with [WSStatus.InstanceNotFound]($bentley) if [[BriefcaseQuery.byId]] is used and a Briefcase with the specified id could not be found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new BriefcaseQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying briefcases for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const id = query.getId();
        const briefcases = await this._handler.getInstances(requestContext, Briefcase, this.getRelativeUrl(iModelId, id), query.getQueryOptions());
        requestContext.enter();
        for (const briefcase of briefcases) {
            briefcase.iModelId = iModelId;
        }
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried briefcases for iModel", () => ({ iModelId, count: briefcases.length }));
        return briefcases;
    }
    /** Download the latest copy of master file. This only downloads the file and does not write the [[Briefcase]] id into it. Use [IModelDb.open]($backend) instead if you want to get a Briefcase file you can work with.
     * This method does not work on the browser. Directory containing the Briefcase file is created if it does not exist. If there is an error during download, any partially downloaded file is deleted from disk.
     * @param requestContext The client request context
     * @param briefcase Briefcase to download. This needs to include a download link. See [[BriefcaseQuery.selectDownloadUrl]].
     * @param path Path where briefcase file should be downloaded, including filename.
     * @param progressCallback Callback for tracking progress.
     * @param cancelRequest Request used to cancel download
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.NotSupportedInBrowser]($bentley) if called in a browser.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.FileHandlerNotSet]($bentley) if [[FileHandler]] instance was not set for [[IModelClient]].
     * @throws [[ResponseError]] if the briefcase cannot be downloaded.
     */
    async download(requestContext, briefcase, path, progressCallback, cancelRequest) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Started downloading briefcase", () => (Object.assign(Object.assign({}, briefcase), { path })));
        Errors_1.ArgumentCheck.defined("briefcase", briefcase);
        Errors_1.ArgumentCheck.defined("path", path);
        if (typeof window !== "undefined")
            throw Errors_1.IModelHubClientError.browser();
        if (!this._fileHandler)
            throw Errors_1.IModelHubClientError.fileHandler();
        if (!briefcase.downloadUrl)
            throw Errors_1.IModelHubClientError.missingDownloadUrl("briefcase");
        await this._fileHandler.downloadFile(requestContext, briefcase.downloadUrl, path, parseInt(briefcase.fileSize, 10), progressCallback, cancelRequest);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Finished downloading briefcase", () => (Object.assign(Object.assign({}, briefcase), { path })));
    }
}
exports.BriefcaseHandler = BriefcaseHandler;
//# sourceMappingURL=Briefcases.js.map