"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckpointV2Handler = exports.CheckpointV2Query = exports.CheckpointV2 = exports.CheckpointV2ErrorId = exports.CheckpointV2State = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/** [[CheckpointV2]] creation state.
 * @internal
 */
var CheckpointV2State;
(function (CheckpointV2State) {
    /** CheckpointV2 creation is in progress. */
    CheckpointV2State[CheckpointV2State["InProgress"] = 0] = "InProgress";
    /** CheckpointV2 creation was successful. */
    CheckpointV2State[CheckpointV2State["Successful"] = 1] = "Successful";
    /** CheckpointV2 creation failed. */
    CheckpointV2State[CheckpointV2State["Failed"] = 2] = "Failed";
    /** CheckpointV2 is not generated. It was never created or was deleted. This state is set by the server. It is possible to recreate CheckpointV2. */
    CheckpointV2State[CheckpointV2State["NotGenerated"] = 3] = "NotGenerated";
})(CheckpointV2State = exports.CheckpointV2State || (exports.CheckpointV2State = {}));
/** [[CheckpointV2]] generation error id.
 * @internal
 */
var CheckpointV2ErrorId;
(function (CheckpointV2ErrorId) {
    CheckpointV2ErrorId[CheckpointV2ErrorId["UnknownError"] = 0] = "UnknownError";
    CheckpointV2ErrorId[CheckpointV2ErrorId["FileDownloadError"] = 1] = "FileDownloadError";
    CheckpointV2ErrorId[CheckpointV2ErrorId["FileUploadError"] = 2] = "FileUploadError";
    CheckpointV2ErrorId[CheckpointV2ErrorId["FileOpenError"] = 3] = "FileOpenError";
    CheckpointV2ErrorId[CheckpointV2ErrorId["ApplyChangeSetError"] = 4] = "ApplyChangeSetError";
    CheckpointV2ErrorId[CheckpointV2ErrorId["TimeOut"] = 5] = "TimeOut";
})(CheckpointV2ErrorId = exports.CheckpointV2ErrorId || (exports.CheckpointV2ErrorId = {}));
/**
 * Checkpoint is a copy of the master file, that is intended to be read-only and reduces amount of merging required to get an iModel to a specific previous state.
 * [[CheckpointV2]] is stored as a set of binary blocks, while [[Checkpoint]] is a single binary file.
 * @internal
 */
let CheckpointV2 = class CheckpointV2 extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ChangeSetId")
], CheckpointV2.prototype, "changeSetId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.State")
], CheckpointV2.prototype, "state", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasContainer].relatedInstance[ContainerAccessKey].properties.Account")
], CheckpointV2.prototype, "containerAccessKeyAccount", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasContainer].relatedInstance[ContainerAccessKey].properties.Container")
], CheckpointV2.prototype, "containerAccessKeyContainer", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasContainer].relatedInstance[ContainerAccessKey].properties.SAS")
], CheckpointV2.prototype, "containerAccessKeySAS", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasContainer].relatedInstance[ContainerAccessKey].properties.DbName")
], CheckpointV2.prototype, "containerAccessKeyDbName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasCheckpointV2FailureInfo](direction:forward).relatedInstance[CheckpointV2FailureInfo].properties.ErrorId")
], CheckpointV2.prototype, "failureInfoErrorId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasCheckpointV2FailureInfo](direction:forward).relatedInstance[CheckpointV2FailureInfo].properties.FailedChangeSetId")
], CheckpointV2.prototype, "failureInfoFailedChangeSetId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasCheckpointV2FailureInfo](direction:forward).relatedInstance[CheckpointV2FailureInfo].properties.StartDate")
], CheckpointV2.prototype, "failureInfoStartDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasCheckpointV2FailureInfo](direction:forward).relatedInstance[CheckpointV2FailureInfo].properties.FailureDate")
], CheckpointV2.prototype, "failureInfoFailureDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasCheckpointV2FailureInfo](direction:forward).relatedInstance[CheckpointV2FailureInfo].properties.JobId")
], CheckpointV2.prototype, "failureInfoJobId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasCheckpointV2FailureInfo](direction:forward).relatedInstance[CheckpointV2FailureInfo].properties.JobRunDurationMS")
], CheckpointV2.prototype, "failureInfoJobRunDurationMS", void 0);
CheckpointV2 = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.CheckpointV2", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], CheckpointV2);
exports.CheckpointV2 = CheckpointV2;
/**
 * Query object for getting [[CheckpointV2]]s. You can use this to modify the [[CheckpointV2Handler.get]] results.
 * @internal
 */
class CheckpointV2Query extends itwin_client_1.WsgQuery {
    /** Query will return [[CheckpointV2]] with specified [[ChangeSet]] id.
     * @returns This query.
     */
    byChangeSetId(changeSetId) {
        Errors_1.ArgumentCheck.validChangeSetId("changeSetId", changeSetId, true);
        this.addFilter(`ChangeSetId+eq+'${changeSetId}'`, "and");
        return this;
    }
    /** Query will return closest [[CheckpointV2]] to target [[ChangeSet]] that does not exceed the specified ChangeSet.
     * This query returns a closest CheckpointV2 that will reach target ChangeSet by only merging forward. This resets all previously set filters.
     * @returns This query.
     */
    precedingCheckpointV2(targetChangeSetId) {
        this.filter(`PrecedingCheckpointV2-backward-ChangeSet.Id+eq+'${targetChangeSetId}'`);
        return this;
    }
    /** Query will return [[CheckpointV2]] with specified state.
     * @returns This query.
     */
    byState(state) {
        this.addFilter(`State+eq+${state}`, "and");
        return this;
    }
    /** Query will additionally select [[CheckpointV2]] container access key to download [[CheckpointV2]] blocks.
     * @returns This query.
     */
    selectContainerAccessKey() {
        HubQuery_1.addSelectContainerAccessKey(this._query);
        return this;
    }
    /** Query will additionally select failure info for failed [[CheckpointV2]].
     * @returns This query.
     */
    selectFailureInfo() {
        if (!this._query.$select)
            this._query.$select = "*";
        this._query.$select += ",HasCheckpointV2FailureInfo-forward-CheckpointV2FailureInfo.*";
        return this;
    }
}
exports.CheckpointV2Query = CheckpointV2Query;
/**
 * Handler for managing [[CheckpointV2]]s. Use [[IModelClient.checkpointsV2]] to get an instance of this class.
 * In most cases, you should use [BriefcaseDb]($backend) methods instead.
 * @internal
 */
class CheckpointV2Handler {
    /** Constructor for CheckpointV2Handler. Use [[IModelClient]] instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for [[CheckpointV2]] requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcaseId Id of the checkpoint.
     * @internal
     */
    getRelativeUrl(iModelId, checkpointId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/CheckpointV2/${checkpointId || ""}`;
    }
    /** Get the [[CheckpointV2]]s.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried [[CheckpointV2]]s or select different data from them.
     * @returns Checkpoints that match the query.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new CheckpointV2Query()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying CheckpointsV2 for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const checkpoints = await this._handler.getInstances(requestContext, CheckpointV2, this.getRelativeUrl(iModelId), query.getQueryOptions());
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried CheckpointsV2 for iModel", () => ({ iModelId, count: checkpoints.length }));
        return checkpoints;
    }
    /** Create a [[CheckpointV2]] for the specified iModel.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param checkpoint [[CheckpointV2]] instance to create. Requires changeSetId to be set. [[CheckpointV2]] is always created with state 'InProgress'.
     * @returns The created [[CheckpointV2]] instance from iModelHub. Container AccessKey is always returned.
     * @throws [[IModelHubError]] with [IModelHubStatus.CheckpointAlreadyExists]($bentley) if [[CheckpointV2]] for a given [[ChangeSet]] was already created.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async create(requestContext, iModelId, checkpoint) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Creating CheckpointV2 for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.defined("checkpoint", checkpoint);
        Errors_1.ArgumentCheck.validChangeSetId("checkpoint.changeSetId", checkpoint.changeSetId, false);
        checkpoint = await this._handler.postInstance(requestContext, CheckpointV2, this.getRelativeUrl(iModelId), checkpoint);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Created CheckpointV2 for iModel", () => ({ iModelId, checkpoint }));
        return checkpoint;
    }
    /** Update the [[CheckpointV2]] of an iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param checkpoint [[CheckpointV2]] to update. Requires wsgId set to existing [[CheckpointV2]] instance.
     * @returns Updated [[CheckpointV2]] instance from iModelHub. Container AccessKey is returned if [[CheckpointV2]] state is 'InProgress'.
     * @throws [[IModelHubError]] with [IModelHubStatus.CheckpointDoesNotExist]($bentley) if [[CheckpointV2]] with a given wsgId does not exist.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async update(requestContext, iModelId, checkpoint) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Updating CheckpointV2 for iModel", () => ({ iModelId, checkpoint }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.defined("checkpoint", checkpoint);
        const checkpointId = parseInt(checkpoint.wsgId, 10);
        Errors_1.ArgumentCheck.definedNumber("checkpoint.wsgId", checkpointId);
        const updatedCheckpoint = await this._handler.postInstance(requestContext, CheckpointV2, this.getRelativeUrl(iModelId, checkpointId), checkpoint);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Updated CheckpointV2 for iModel", () => ({ iModelId, updatedCheckpoint }));
        return updatedCheckpoint;
    }
}
exports.CheckpointV2Handler = CheckpointV2Handler;
//# sourceMappingURL=CheckpointsV2.js.map