"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelHubClient = void 0;
const IModelClient_1 = require("../IModelClient");
const BaseHandler_1 = require("./BaseHandler");
const Permissions_1 = require("./Permissions");
/**
 * Class that allows access to different iModelHub class handlers. Handlers should be accessed through an instance of this class, rather than constructed directly.
 * @public
 */
class IModelHubClient extends IModelClient_1.IModelClient {
    /**
     * Create an instance of IModelHubClient.
     * @param fileHandler File handler to handle file upload/download and file system operations.
     * @param iModelBaseHandler WSG Client for the iModelHub operations.
     * @param applicationVersion Version of the current application.
     */
    constructor(fileHandler, iModelBaseHandler = new BaseHandler_1.IModelBaseHandler(), applicationVersion) {
        super(iModelBaseHandler, fileHandler, applicationVersion);
    }
    /**
     * Get the handler for permissions.
     * @internal
     */
    get permissions() {
        return new Permissions_1.PermissionHandler(this._handler);
    }
}
exports.IModelHubClient = IModelHubClient;
//# sourceMappingURL=Client.js.map