"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventHandler = exports.EventSubscriptionHandler = exports.EventSAS = exports.EventSubscription = exports.ParseEvent = exports.constructorFromEventType = exports.CheckpointV2CreatedEvent = exports.CheckpointCreatedEvent = exports.VersionEvent = exports.IModelDeletedEvent = exports.BriefcaseDeletedEvent = exports.AllCodesDeletedEvent = exports.CodeEvent = exports.ChangeSetPrePushEvent = exports.ChangeSetPostPushEvent = exports.AllLocksDeletedEvent = exports.LockEvent = exports.BriefcaseEvent = exports.IModelHubEvent = exports.IModelHubEventType = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Codes_1 = require("./Codes");
const Errors_1 = require("./Errors");
const EventsBase_1 = require("./EventsBase");
const Locks_1 = require("./Locks");
/* eslint-disable @typescript-eslint/no-shadow */
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/** Type of [[IModelHubEvent]]. Event type is used to define which events you wish to receive from your [[EventSubscription]]. See [[EventSubscriptionHandler.create]] and [[EventSubscriptionHandler.update]].
 * @public
 */
var IModelHubEventType;
(function (IModelHubEventType) {
    /** Sent when one or more [[Lock]]s are updated. See [[LockEvent]].
     * @internal
     */
    IModelHubEventType["LockEvent"] = "LockEvent";
    /** Sent when all [[Lock]]s for a [[Briefcase]] are deleted. See [[AllLocksDeletedEvent]].
     * @internal
     */
    IModelHubEventType["AllLocksDeletedEvent"] = "AllLocksDeletedEvent";
    /** Sent when a [[ChangeSet]] is successfully pushed. See [[ChangeSetPostPushEvent]]. */
    IModelHubEventType["ChangeSetPostPushEvent"] = "ChangeSetPostPushEvent";
    /** Sent when a [[ChangeSet]] push has started. See [[ChangeSetPrePushEvent]]. */
    IModelHubEventType["ChangeSetPrePushEvent"] = "ChangeSetPrePushEvent";
    /** Sent when one or more [Code]($common)s are updated. See [[CodeEvent]].
     * @internal
     */
    IModelHubEventType["CodeEvent"] = "CodeEvent";
    /** Sent when all [Code]($common)s for a [[Briefcase]] are deleted. See [[AllCodesDeletedEvent]].
     * @internal
     */
    IModelHubEventType["AllCodesDeletedEvent"] = "AllCodesDeletedEvent";
    /** Sent when a [[Briefcase]] is deleted. See [[BriefcaseDeletedEvent]].
     * @internal
     */
    IModelHubEventType["BriefcaseDeletedEvent"] = "BriefcaseDeletedEvent";
    /** Sent when an iModel is deleted. See [[iModelDeletedEvent]]. */
    IModelHubEventType["iModelDeletedEvent"] = "iModelDeletedEvent";
    /** Sent when a new named [[Version]] is created. See [[VersionEvent]]. */
    IModelHubEventType["VersionEvent"] = "VersionEvent";
    /**
     * Sent when a new [[Checkpoint]] is generated. See [[CheckpointCreatedEvent]].
     * @internal
     */
    IModelHubEventType["CheckpointCreatedEvent"] = "CheckpointCreatedEvent";
    /**
     * Sent when a new [[CheckpointV2]] is generated. See [[CheckpointV2CreatedEvent]].
     * @internal
     */
    IModelHubEventType["CheckpointV2CreatedEvent"] = "CheckpointV2CreatedEvent";
})(IModelHubEventType = exports.IModelHubEventType || (exports.IModelHubEventType = {}));
/** Base type for all iModelHub events.
 * @public
 */
class IModelHubEvent extends EventsBase_1.IModelHubBaseEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.iModelId = this.eventTopic;
    }
}
exports.IModelHubEvent = IModelHubEvent;
/** Base type for iModelHub events that have BriefcaseId.
 * @public
 */
class BriefcaseEvent extends IModelHubEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.briefcaseId = obj.BriefcaseId;
    }
}
exports.BriefcaseEvent = BriefcaseEvent;
/** Sent when one or more [[Lock]]s are updated. Lock updates can be very frequent, so it's recommended to not to subscribe to LockEvents, if it's not necessary.
 * @internal
 */
class LockEvent extends BriefcaseEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.lockType = Locks_1.LockType[obj.LockType];
        this.lockLevel = Locks_1.LockLevel[obj.LockLevel];
        this.objectIds = obj.ObjectIds.map((value) => bentleyjs_core_1.Id64.fromJSON(value));
        this.releasedWithChangeSet = obj.ReleasedWithChangeSet;
    }
}
exports.LockEvent = LockEvent;
/** Sent when all [[Lock]]s for a [[Briefcase]] are deleted. Can occur when calling [[LockHandler.deleteAll]] or [[BriefcaseHandler.delete]].
 * @internal
 */
class AllLocksDeletedEvent extends BriefcaseEvent {
}
exports.AllLocksDeletedEvent = AllLocksDeletedEvent;
/** Sent when a [[ChangeSet]] is successfully pushed. See [[ChangeSetHandler.create]]. It's sent when a new [[ChangeSet]] is successfully pushed to an iModel. See [[ChangeSetPrePushEvent]] for the event indicating the start of a ChangeSet push.
 * @public
 */
class ChangeSetPostPushEvent extends BriefcaseEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.changeSetId = obj.ChangeSetId;
        this.changeSetIndex = obj.ChangeSetIndex;
    }
}
exports.ChangeSetPostPushEvent = ChangeSetPostPushEvent;
/** Sent when a [[ChangeSet]] push has started. See [[ChangeSetHandler.create]]. ChangeSetPrePushEvent indicates that iModelHub allowed one of the [[Briefcase]]s to push a ChangeSet and all other push attempts will fail, until this push times out or succeeds. See [[ChangeSetPostPushEvent]] for an event indicating a successful push.
 * @public
 */
class ChangeSetPrePushEvent extends IModelHubEvent {
}
exports.ChangeSetPrePushEvent = ChangeSetPrePushEvent;
/** Sent when one or more [Code]($common)s are updated. See [[CodeHandler.update]]. Code updates can be very frequent, so it's recommended to not to subscribe to CodeEvents, if it's not necessary.
 * @internal
 */
class CodeEvent extends BriefcaseEvent {
    constructor() {
        super(...arguments);
        /** State Codes were updated to. */
        this.state = Codes_1.CodeState.Reserved;
    }
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.codeSpecId = bentleyjs_core_1.Id64.fromJSON(obj.CodeSpecId);
        this.codeScope = obj.CodeScope;
        this.values = obj.Values;
        this.state = obj.State;
    }
}
exports.CodeEvent = CodeEvent;
/** Sent when all [Code]($common)s for a [[Briefcase]] are deleted. Can occur when calling [[CodeHandler.deleteAll]] or [[BriefcaseHandler.delete]].
 * @internal
 */
class AllCodesDeletedEvent extends BriefcaseEvent {
}
exports.AllCodesDeletedEvent = AllCodesDeletedEvent;
/** Sent when a [[Briefcase]] is deleted. See [[BriefcaseHandler.delete]].
 * @internal
 */
class BriefcaseDeletedEvent extends BriefcaseEvent {
}
exports.BriefcaseDeletedEvent = BriefcaseDeletedEvent;
/** Sent when an iModel is deleted. See [[IModelHandler.delete]]. [[EventSubscription]] will be deleted 5 minutes after iModel is deleted, removing all events from subscription queues, making it possible for this event to be missed if not retrieved immediately.
 * @public
 */
class IModelDeletedEvent extends IModelHubEvent {
}
exports.IModelDeletedEvent = IModelDeletedEvent;
/** Sent when a new named [[Version]] is created. See [[VersionHandler.create]].
 * @public
 */
class VersionEvent extends IModelHubEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.versionId = obj.VersionId;
        this.versionName = obj.VersionName;
        this.changeSetId = obj.ChangeSetId;
    }
}
exports.VersionEvent = VersionEvent;
/** Sent when a new [[Checkpoint]] is generated. [[Checkpoint]]s can be generated daily when there are new [[ChangeSet]]s pushed or when a new [[Version]] is created.
 * @internal
 */
class CheckpointCreatedEvent extends IModelHubEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.changeSetIndex = obj.ChangeSetIndex;
        this.changeSetId = obj.ChangeSetId;
        this.versionId = obj.VersionId;
    }
}
exports.CheckpointCreatedEvent = CheckpointCreatedEvent;
/** Sent when a new [[CheckpointV2]] is generated. [[CheckpointV2]] might be created for every [[ChangeSet]].
 * @internal
 */
class CheckpointV2CreatedEvent extends IModelHubEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.changeSetIndex = obj.ChangeSetIndex;
        this.changeSetId = obj.ChangeSetId;
        this.versionId = obj.VersionId;
    }
}
exports.CheckpointV2CreatedEvent = CheckpointV2CreatedEvent;
/** Get constructor from EventType name.
 * @internal
 */
function constructorFromEventType(type) {
    switch (type) {
        case IModelHubEventType.LockEvent:
            return LockEvent;
        case IModelHubEventType.AllLocksDeletedEvent:
            return AllLocksDeletedEvent;
        case IModelHubEventType.ChangeSetPostPushEvent:
            return ChangeSetPostPushEvent;
        case IModelHubEventType.ChangeSetPrePushEvent:
            return ChangeSetPrePushEvent;
        case IModelHubEventType.CodeEvent:
            return CodeEvent;
        case IModelHubEventType.AllCodesDeletedEvent:
            return AllCodesDeletedEvent;
        case IModelHubEventType.BriefcaseDeletedEvent:
            return BriefcaseDeletedEvent;
        case IModelHubEventType.iModelDeletedEvent:
            return IModelDeletedEvent;
        case IModelHubEventType.VersionEvent:
            return VersionEvent;
        case IModelHubEventType.CheckpointCreatedEvent:
            return CheckpointCreatedEvent;
        case IModelHubEventType.CheckpointV2CreatedEvent:
            return CheckpointV2CreatedEvent;
    }
}
exports.constructorFromEventType = constructorFromEventType;
/** Parse [[IModelHubEvent]] from response object.
 * @param response Response object to parse.
 * @returns Appropriate event object.
 * @internal
 */
// eslint-disable-next-line @typescript-eslint/naming-convention
function ParseEvent(response) {
    const constructor = constructorFromEventType(response.header["content-type"]);
    const event = new constructor();
    event.fromJson(response.body);
    return event;
}
exports.ParseEvent = ParseEvent;
/** Subscription to receive [[IModelHubEvent]]s. Each subscription has a separate queue for events that it hasn't read yet. Subscriptions are deleted, if they are inactive for an hour. Use wsgId of this instance for the methods that require subscriptionId. See [[EventSubscriptionHandler]].
 * @public
 */
let EventSubscription = class EventSubscription extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.EventTypes")
], EventSubscription.prototype, "eventTypes", void 0);
EventSubscription = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.EventSubscription", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], EventSubscription);
exports.EventSubscription = EventSubscription;
/** Shared access signature token for getting [[IModelHubEvent]]s. It's used to authenticate for [[EventHandler.getEvent]]. To receive an instance call [[EventHandler.getSASToken]].
 * @public
 */
let EventSAS = class EventSAS extends EventsBase_1.BaseEventSAS {
};
EventSAS = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.EventSAS", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], EventSAS);
exports.EventSAS = EventSAS;
/** Handler for managing [[EventSubscription]]s. Use [[EventHandler.Subscriptions]] to get an instance of this class.
 * @public
 */
class EventSubscriptionHandler {
    /** Constructor for EventSubscriptionHandler.
     * @param handler Handler for WSG requests.
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for EventSubscription requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param instanceId Id of the subscription.
     */
    getRelativeUrl(iModelId, instanceId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/EventSubscription/${instanceId || ""}`;
    }
    async create(requestContext, iModelId, events) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Creating event subscription on iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.nonEmptyArray("events", events);
        let subscription = new EventSubscription();
        subscription.eventTypes = events;
        subscription = await this._handler.postInstance(requestContext, EventSubscription, this.getRelativeUrl(iModelId), subscription);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Created event subscription on iModel", () => ({ iModelId }));
        return subscription;
    }
    /** Update an [[EventSubscription]]. Can change the [[EventType]]s specified in the subscription. Must be a valid subscription that was previously created with [[EventSubscriptionHandler.create]] that hasn't expired.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param subscription Updated EventSubscription.
     * @return EventSubscription instance from iModelHub after update.
     * @throws [[IModelHubError]] with [IModelHubStatus.EventSubscriptionDoesNotExist]($bentley) if [[EventSubscription]] does not exist with the specified subscription.wsgId.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, iModelId, subscription) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Updating event subscription on iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.defined("subscription", subscription);
        Errors_1.ArgumentCheck.validGuid("subscription.wsgId", subscription.wsgId);
        const updatedSubscription = await this._handler.postInstance(requestContext, EventSubscription, this.getRelativeUrl(iModelId, subscription.wsgId), subscription);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Updated event subscription on iModel", () => ({ iModelId }));
        return updatedSubscription;
    }
    /** Delete an [[EventSubscription]].
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param eventSubscriptionId Id of the EventSubscription.
     * @returns Resolves if the EventSubscription has been successfully deleted.
     * @throws [[IModelHubError]] with [IModelHubStatus.EventSubscriptionDoesNotExist]($bentley) if EventSubscription does not exist with the specified subscription.wsgId.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async delete(requestContext, iModelId, eventSubscriptionId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Deleting event subscription ${eventSubscriptionId} from iModel`, () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.validGuid("eventSubscriptionId", eventSubscriptionId);
        await this._handler.delete(requestContext, this.getRelativeUrl(iModelId, eventSubscriptionId));
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Deleted event subscription ${eventSubscriptionId} from iModel`, () => ({ iModelId }));
    }
}
exports.EventSubscriptionHandler = EventSubscriptionHandler;
/** Handler for receiving [[IModelHubEvent]]s. Use [[IModelClient.Events]] to get an instance of this class.
 * @public
 */
class EventHandler extends EventsBase_1.EventBaseHandler {
    /** Constructor for EventHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        super();
        this._handler = handler;
    }
    /** Get a handler for managing [[EventSubscription]]s. */
    get subscriptions() {
        if (!this._subscriptionHandler) {
            this._subscriptionHandler = new EventSubscriptionHandler(this._handler);
        }
        return this._subscriptionHandler;
    }
    /** Get relative url for EventSAS requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     */
    getEventSASRelativeUrl(iModelId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/EventSAS/`;
    }
    /** Get event SAS Token. Used to authenticate for [[EventHandler.getEvent]].
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @return SAS Token to connect to the topic.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async getSASToken(requestContext, iModelId) {
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Getting event SAS token from iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const eventSAS = await this._handler.postInstance(requestContext, EventSAS, this.getEventSASRelativeUrl(iModelId), new EventSAS());
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Got event SAS token from iModel", () => ({ iModelId }));
        return eventSAS;
    }
    /** Get absolute url for event requests.
     * @param baseAddress Base address for the serviceBus.
     * @param subscriptionId Id of the subscription.
     * @param timeout Optional timeout for long polling.
     */
    getEventUrl(baseAddress, subscriptionId, timeout) {
        let url = `${baseAddress}/Subscriptions/${subscriptionId}/messages/head`;
        if (timeout) {
            url = `${url}?timeout=${timeout}`;
        }
        return url;
    }
    /** Get [[IModelHubEvent]] from the [[EventSubscription]]. You can use long polling timeout, to have requests return when events are available (or request times out), rather than returning immediately when no events are found.
     * @param requestContext The client request context
     * @param sasToken SAS Token used to authenticate. See [[EventSAS.sasToken]].
     * @param baseAddress Address for the events. See [[EventSAS.baseAddress]].
     * @param subscriptionId Id of the subscription to the topic. See [[EventSubscription]].
     * @param timeout Optional timeout duration in seconds for request, when using long polling.
     * @return IModelHubEvent if it exists, undefined otherwise.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     * @throws [ResponseError]($itwin-client) if request has failed.
     */
    async getEvent(requestContext, sasToken, baseAddress, subscriptionId, timeout) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Getting event from subscription", () => ({ subscriptionId }));
        Errors_1.ArgumentCheck.defined("sasToken", sasToken);
        Errors_1.ArgumentCheck.defined("baseAddress", baseAddress);
        Errors_1.ArgumentCheck.validGuid("subscriptionId", subscriptionId);
        const options = await this.getEventRequestOptions(EventsBase_1.GetEventOperationToRequestType.GetDestructive, sasToken, timeout);
        const result = await itwin_client_1.request(requestContext, this.getEventUrl(baseAddress, subscriptionId, timeout), options);
        requestContext.enter();
        if (result.status === 204) {
            bentleyjs_core_1.Logger.logTrace(loggerCategory, "No events found on subscription", () => ({ subscriptionId }));
            return undefined;
        }
        const event = ParseEvent(result);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Got event from subscription", () => ({ subscriptionId }));
        return event;
    }
    /** Create a listener for long polling events from an [[EventSubscription]]. When event is received from the subscription, every registered listener callback is called. This continuously waits for events until all created listeners for that subscriptionId are deleted. [[EventSAS]] token expirations are handled automatically, [AccessToken]($itwin-client) expiration is handled by calling authenticationCallback to get a new token.
     * @param requestContext The client request context
     * @param authenticationCallback Callback used to get AccessToken. Only the first registered authenticationCallback for this subscriptionId will be used.
     * @param subscriptionId Id of EventSubscription.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param listener Callback that is called when an [[IModelHubEvent]] is received.
     * @returns Function that deletes the created listener.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     */
    createListener(requestContext, authenticationCallback, subscriptionId, iModelId, listener) {
        requestContext.enter();
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.defined("authenticationCallback", authenticationCallback);
        Errors_1.ArgumentCheck.validGuid("subscriptionId", subscriptionId);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const subscription = new EventsBase_1.ListenerSubscription();
        subscription.authenticationCallback = authenticationCallback;
        subscription.getEvent = async (sasToken, baseAddress, id, timeout) => this.getEvent(requestContext, sasToken, baseAddress, id, timeout);
        subscription.getSASToken = async (requestContextArg) => this.getSASToken(requestContextArg, iModelId);
        subscription.id = subscriptionId;
        return EventsBase_1.EventListener.create(subscription, listener);
    }
}
exports.EventHandler = EventHandler;
//# sourceMappingURL=Events.js.map