/** @packageDocumentation
 * @module iModelHubClient
 */
import { GuidString } from "@bentley/bentleyjs-core";
import { RequestQueryOptions, WsgQuery } from "@bentley/itwin-client";
/** Query for instances with string based instance ids.
 * @public
 */
export declare class StringIdQuery extends WsgQuery {
    /** @internal */
    protected _byId?: string;
    /**
     * Query single instance by its id.
     * @param id Id of the instance to query.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if id is undefined or it is not a valid [GuidString]($bentley) value.
     */
    byId(id: string): this;
    /** @internal */
    protected checkValue(id: string): void;
    /**
     * Used by iModelHub handlers to get the id that is queried.
     * @internal
     */
    getId(): string | undefined;
}
/** Query for instances with Guid based instance ids.
 * @public
 */
export declare class InstanceIdQuery extends WsgQuery {
    /** @internal */
    protected _byId?: GuidString;
    /**
     * Query single instance by its id.
     * @param id Id of the instance to query.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if id is undefined or it is not a valid [GuidString]($bentley) value.
     */
    byId(id: GuidString): this;
    /**
     * Used by iModelHub handlers to get the id that is queried.
     * @internal
     */
    getId(): string | undefined;
}
/**
 * Add select for the download URL to the query.
 * @internal
 */
export declare function addSelectFileAccessKey(query: RequestQueryOptions): void;
/**
 * Add select for the ContainerAccessKey to the query that allows
 * to read/download iModel in blocks incrementally.
 * @internal
 */
export declare function addSelectContainerAccessKey(query: RequestQueryOptions): void;
/**
 * Add select for the application data to the query.
 * @internal
 */
export declare function addSelectApplicationData(query: RequestQueryOptions): void;
//# sourceMappingURL=HubQuery.d.ts.map