/** @packageDocumentation
 * @module iModelHubClient
 */
import { GuidString } from "@bentley/bentleyjs-core";
import { AuthorizedClientRequestContext, WsgInstance } from "@bentley/itwin-client";
import { IModelBaseHandler } from "./BaseHandler";
/**
 * iModelHub iModel Permissions
 * @internal
 */
export declare class IModelPermissions extends WsgInstance {
    /** Allows to view iModel in web browser, but does not allow to get its local copy. */
    webView?: string;
    /** Allows to open and view an iModel only in read-only state. */
    read?: string;
    /** Allows to make changes to an iModel and create/modify named versions. */
    write?: string;
    /** Allows to manage locks, codes or local copies for the entire iModel. */
    manage?: string;
}
/**
 * Handler for managing permissions. Use [[IModelHubClient.permissions]] to get an instance of this class.
 * @internal
 */
export declare class PermissionHandler {
    private _handler;
    /**
     * Constructor for PermissionHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler: IModelBaseHandler);
    /**
     * Get user permissions of an specified iModel.
     * Use [[PermissionHandler.getContextPermissions]] to check CONNECT context level permissions.
     * @param requestContext The client request context.
     * @param imodelId Id of the specified iModel.
     */
    getiModelPermissions(requestContext: AuthorizedClientRequestContext, imodelId: GuidString): Promise<IModelPermissions>;
}
//# sourceMappingURL=Permissions.d.ts.map