"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PermissionHandler = exports.IModelPermissions = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * iModelHub iModel Permissions
 * @internal
 */
let IModelPermissions = class IModelPermissions extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.WebView")
], IModelPermissions.prototype, "webView", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Read")
], IModelPermissions.prototype, "read", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Write")
], IModelPermissions.prototype, "write", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Manage")
], IModelPermissions.prototype, "manage", void 0);
IModelPermissions = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.Permission", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], IModelPermissions);
exports.IModelPermissions = IModelPermissions;
/**
 * Handler for managing permissions. Use [[IModelHubClient.permissions]] to get an instance of this class.
 * @internal
 */
class PermissionHandler {
    /**
     * Constructor for PermissionHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /**
     * Get user permissions of an specified iModel.
     * Use [[PermissionHandler.getContextPermissions]] to check CONNECT context level permissions.
     * @param requestContext The client request context.
     * @param imodelId Id of the specified iModel.
     */
    async getiModelPermissions(requestContext, imodelId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying permissions for iModel", () => ({ iModelId: imodelId }));
        const permissions = await this._handler.getInstances(requestContext, IModelPermissions, `/Repositories/iModel--${imodelId}/iModelScope/Permission`);
        return permissions[0];
    }
}
exports.PermissionHandler = PermissionHandler;
//# sourceMappingURL=Permissions.js.map