"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelHandler = exports.IModelsHandler = exports.DefaultIModelCreateOptionsProvider = exports.IModelQuery = exports.SeedFile = exports.IModelType = exports.InitializationState = exports.HubIModel = void 0;
const deepAssign = require("deep-assign");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * HubIModel represents an iModel on iModelHub. Getting a valid HubIModel instance from iModelHub is required for majority of iModelHub method calls, as wsgId of this object needs to be passed as iModelId argument to those methods.
 *
 * For iModel representation in iModel.js, see [IModel]($common). For the file that is used for that iModel, see [BriefcaseDb]($backend).
 * @public
 */
let HubIModel = class HubIModel extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "instanceId")
], HubIModel.prototype, "id", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Description")
], HubIModel.prototype, "description", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Name")
], HubIModel.prototype, "name", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.UserCreated")
], HubIModel.prototype, "userCreated", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CreatedDate")
], HubIModel.prototype, "createdDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Initialized")
], HubIModel.prototype, "initialized", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.iModelTemplate")
], HubIModel.prototype, "iModelTemplate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Type")
], HubIModel.prototype, "iModelType", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Extent")
], HubIModel.prototype, "extent", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Secured")
], HubIModel.prototype, "secured", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.DataLocationId")
], HubIModel.prototype, "dataLocationId", void 0);
HubIModel = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "ContextScope.iModel", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], HubIModel);
exports.HubIModel = HubIModel;
/** Initialization state of seed file. Can be queried with [[IModelHandler.getInitializationState]]. See [iModel creation]($docs/learning/iModelHub/iModels/CreateiModel.md).
 * @public
 */
var InitializationState;
(function (InitializationState) {
    /** Initialization was successful. */
    InitializationState[InitializationState["Successful"] = 0] = "Successful";
    /** Initialization has not started, seed file has not yet been uploaded. */
    InitializationState[InitializationState["NotStarted"] = 1] = "NotStarted";
    /** Initialization has been scheduled and has not completed yet. */
    InitializationState[InitializationState["Scheduled"] = 2] = "Scheduled";
    /** Initialization failed with a generic error. */
    InitializationState[InitializationState["Failed"] = 3] = "Failed";
    /** Initialization failed due to file having outdated schemas. */
    InitializationState[InitializationState["OutdatedFile"] = 4] = "OutdatedFile";
    /** Initialization failed due to file having [[Code]] values that are too long. */
    InitializationState[InitializationState["CodeTooLong"] = 5] = "CodeTooLong";
    /** Initialization failed due to file being a [[Briefcase]]. Only standalone and master files are supported for iModel creation, see [BriefcaseId]($backend). */
    InitializationState[InitializationState["SeedFileIsBriefcase"] = 6] = "SeedFileIsBriefcase";
})(InitializationState = exports.InitializationState || (exports.InitializationState = {}));
/** iModel type
 * @public
 */
var IModelType;
(function (IModelType) {
    /** iModel has no type. */
    IModelType[IModelType["Undefined"] = 0] = "Undefined";
    /** iModel contains metadata used for other iModel creation. */
    IModelType[IModelType["Library"] = 1] = "Library";
})(IModelType = exports.IModelType || (exports.IModelType = {}));
/** SeedFile
 * @internal
 */
let SeedFile = class SeedFile extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "instanceId")
], SeedFile.prototype, "id", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileName")
], SeedFile.prototype, "fileName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileDescription")
], SeedFile.prototype, "fileDescription", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileSize")
], SeedFile.prototype, "fileSize", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileId")
], SeedFile.prototype, "fileId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Index")
], SeedFile.prototype, "index", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.IModelName")
], SeedFile.prototype, "iModelName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.MergedChangeSetId")
], SeedFile.prototype, "mergedChangeSetId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.UserUploaded")
], SeedFile.prototype, "userUploaded", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.UploadedDate")
], SeedFile.prototype, "uploadedDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.IsUploaded")
], SeedFile.prototype, "isUploaded", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.InitializationState")
], SeedFile.prototype, "initializationState", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[FileAccessKey].relatedInstance[AccessKey].properties.DownloadUrl")
], SeedFile.prototype, "downloadUrl", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[FileAccessKey].relatedInstance[AccessKey].properties.UploadUrl")
], SeedFile.prototype, "uploadUrl", void 0);
SeedFile = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.SeedFile", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], SeedFile);
exports.SeedFile = SeedFile;
/** Query object for getting SeedFiles. You can use this to modify the query. See [[SeedFileHandler.get]]. */
class SeedFileQuery extends HubQuery_1.InstanceIdQuery {
    /**
     * Query will additionally select SeedFile file download URL.
     * @returns This query.
     */
    selectDownloadUrl() {
        HubQuery_1.addSelectFileAccessKey(this._query);
        return this;
    }
    /**
     * Change the order to latest changesets first in the query.
     * @returns This query.
     */
    latest() {
        this._query.$orderby = "Index+desc";
        return this;
    }
}
/** Handler for all methods related to @see SeedFile instances. */
class SeedFileHandler {
    /** Constructor for SeedFileHandler. Should use @see IModelHandler instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @param fileHandler Handler for file system.
     */
    constructor(handler, fileHandler) {
        this._handler = handler;
        this._fileHandler = fileHandler;
    }
    /** Get relative url for SeedFile requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param fileId Id of the Seed File.
     */
    getRelativeUrl(iModelId, fileId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/SeedFile/${fileId || ""}`;
    }
    /** Get the seed files given the id of the iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried SeedFiles or select different data from them.
     * @returns Resolves to the seed file.
     */
    async get(requestContext, iModelId, query = new SeedFileQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Started querying seed files", () => ({ iModelId }));
        const seedFiles = await this._handler.getInstances(requestContext, SeedFile, this.getRelativeUrl(iModelId, query.getId()), query.getQueryOptions());
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Finished querying seed files", () => ({ iModelId, count: seedFiles.length }));
        return seedFiles;
    }
    /** Upload the seed file. Use [[confirmUploadSeedFile]] to confirm the completion of the upload.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param seedFile Information of the SeedFile to be uploaded.
     * @param seedPath Path of the SeedFile to be uploaded.
     * @param progressCallback Callback for tracking progress.
     */
    async uploadSeedFile(requestContext, iModelId, seedPath, seedFileDescription, progressCallback) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Started uploading seed file", () => ({ iModelId, seedPath }));
        const seedFile = new SeedFile();
        seedFile.fileName = this._fileHandler.basename(seedPath);
        seedFile.fileSize = this._fileHandler.getFileSize(seedPath).toString();
        if (seedFileDescription)
            seedFile.fileDescription = seedFileDescription;
        const createdSeedFile = await this._handler.postInstance(requestContext, SeedFile, this.getRelativeUrl(iModelId), seedFile);
        requestContext.enter();
        await this._fileHandler.uploadFile(requestContext, createdSeedFile.uploadUrl, seedPath, progressCallback);
        requestContext.enter();
        createdSeedFile.uploadUrl = undefined;
        createdSeedFile.downloadUrl = undefined;
        createdSeedFile.isUploaded = true;
        const confirmSeedFile = await this._handler.postInstance(requestContext, SeedFile, this.getRelativeUrl(iModelId, createdSeedFile.id), createdSeedFile);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Finished uploading seed file", () => confirmSeedFile);
        return confirmSeedFile;
    }
}
/**
 * Query object for getting [[HubIModel]] instances. You can use this to modify the [[IModelsHandler.get]] results.
 * @public
 */
class IModelQuery extends HubQuery_1.InstanceIdQuery {
    /**
     * Query iModel by its name.
     * @param name Name of the iModel.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if name is undefined or empty.
     */
    byName(name) {
        Errors_1.ArgumentCheck.defined("name", name);
        this.addFilter(`Name+eq+'${encodeURIComponent(name)}'`);
        return this;
    }
    /**
     * Query iModel by its type.
     * @param iModelType Type of the iModel.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if iModelType is undefined.
     */
    byiModelType(iModelType) {
        Errors_1.ArgumentCheck.defined("iModelType", iModelType);
        this.addFilter(`Type+eq+${iModelType}`);
        return this;
    }
    /**
     * Query iModel by its template.
     * @param type Type of the iModel.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if iModelTemplate is undefined or empty.
     * @internal
     */
    byiModelTemplate(iModelTemplate) {
        Errors_1.ArgumentCheck.defined("iModelTemplate", iModelTemplate, true);
        this.addFilter(`iModelTemplate+eq+'${encodeURIComponent(iModelTemplate)}'`);
        return this;
    }
}
exports.IModelQuery = IModelQuery;
const iModelTemplateEmpty = "Empty";
/**
 * Provider for default IModelCreateOptions, used by IModelHandler and IModelsHandler to set defaults.
 * @internal
 */
class DefaultIModelCreateOptionsProvider {
    /**  Creates an instance of DefaultRequestOptionsProvider and sets up the default options. */
    constructor() {
        this._defaultOptions = {
            timeOutInMilliseconds: 120000,
        };
    }
    /**
     * Augments options with the provider's default values. The options passed in override any defaults where necessary.
     * @param options Options that should be augmented.
     */
    async assignOptions(options) {
        const clonedOptions = Object.assign({}, options);
        deepAssign(options, this._defaultOptions);
        deepAssign(options, clonedOptions); // ensure the supplied options override the defaults
        if (!options.template) // this assignment works incorrectly through deepAssign
            options.template = iModelTemplateEmpty;
    }
    /**
     * Formats iModel template value as a string.
     * @param options Options that have the template value.
     */
    templateToString(options) {
        if (!options.template || options.template === iModelTemplateEmpty)
            return options.template;
        return `${options.template.imodelId}:${options.template.changeSetId || ""}`;
    }
}
exports.DefaultIModelCreateOptionsProvider = DefaultIModelCreateOptionsProvider;
/**
 * Handler for managing [[HubIModel]] instances. Use [[IModelHubClient.IModels]] to get an instance of this handler.
 * @note Use [[IModelHubClient.IModel]] for the preferred single iModel per context workflow.
 * @public
 */
class IModelsHandler {
    /** Constructor for IModelsHandler. Should use @see IModelClient instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @param fileHandler Handler for file system.
     * @note Use [[IModelHubClient.IModel]] for the preferred single iModel per context workflow.
     * @internal
     */
    constructor(handler, fileHandler) {
        this._handler = handler;
        this._fileHandler = fileHandler;
        this._seedFileHandler = new SeedFileHandler(this._handler, this._fileHandler);
    }
    /** Get relative url for iModel requests.
     * @param contextId Id of the context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     */
    getRelativeUrl(contextId, iModelId) {
        return `/Repositories/Context--${this._handler.formatContextIdForUrl(contextId)}/ContextScope/iModel/${iModelId || ""}`;
    }
    /** Get iModels that belong to the specified context.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param query Optional query object to filter the queried iModels or select different data from them.
     * @returns [[HubIModel]] instances that match the query.
     * @throws [WsgError]($itwin-client) with [WSStatus.InstanceNotFound]($bentley) if [[InstanceIdQuery.byId]] is used and an HubIModel with the specified id could not be found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, contextId, query = new IModelQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Started querying iModels in context`, () => ({ contextId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.defined("contextId", contextId); // contextId is a GUID for iModelHub and a JSON representation of an IModelBankAccessContext for iModelBank.
        const imodels = await this._handler.getInstances(requestContext, HubIModel, this.getRelativeUrl(contextId, query.getId()), query.getQueryOptions());
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Finished querying iModels in context`, () => ({ contextId, count: imodels.length }));
        return imodels;
    }
    /** Delete an iModel with specified id from a context. This method is not supported in iModelBank.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param iModelId Id of the iModel to be deleted. See [[HubIModel]].
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel with specified id does not exist.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have DeleteiModel permission.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async delete(requestContext, contextId, iModelId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Started deleting iModel", () => ({ iModelId, contextId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("contextId", contextId);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        if (this._handler.getCustomRequestOptions().isSet) {
            // to add custom request options, request with body is needed.
            const imodel = new HubIModel();
            imodel.id = iModelId;
            imodel.changeState = "deleted";
            await this._handler.deleteInstance(requestContext, this.getRelativeUrl(contextId, iModelId), imodel);
        }
        else {
            await this._handler.delete(requestContext, this.getRelativeUrl(contextId, iModelId));
        }
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Finished deleting iModel", () => ({ iModelId, contextId }));
    }
    /** Create an iModel instance
     * @param requestContext The client request context.
     * @param contextId Id of the iTwin context.
     * @param iModelName Name of the iModel on the Hub.
     * @param description Description of the iModel on the Hub.
     * @param iModelTemplate iModel template.
     * @param iModelType iModel type.
     */
    async createIModelInstance(requestContext, contextId, iModelName, description, iModelTemplate, iModelType, extent) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Creating iModel with name ${iModelName}`, () => ({ contextId }));
        let imodel;
        const iModel = new HubIModel();
        iModel.name = iModelName;
        if (description)
            iModel.description = description;
        if (iModelTemplate)
            iModel.iModelTemplate = iModelTemplate;
        if (extent)
            iModel.extent = extent;
        if (iModelType)
            iModel.iModelType = iModelType;
        try {
            imodel = await this._handler.postInstance(requestContext, HubIModel, this.getRelativeUrl(contextId), iModel);
            requestContext.enter();
            bentleyjs_core_1.Logger.logTrace(loggerCategory, `Created iModel instance with name ${iModelName}`, () => ({ contextId }));
        }
        catch (err) {
            requestContext.enter();
            if (!(err instanceof Errors_1.IModelHubError) || bentleyjs_core_1.IModelHubStatus.iModelAlreadyExists !== err.errorNumber) {
                bentleyjs_core_1.Logger.logWarning(loggerCategory, `Can not create iModel: ${err.message}`, () => ({ contextId }));
                throw err;
            }
            const initialized = err.data.iModelInitialized;
            if (initialized) {
                bentleyjs_core_1.Logger.logWarning(loggerCategory, `Error creating iModel: iModel with name ${iModelName} already exists and is initialized`, () => ({ contextId }));
                throw err;
            }
            bentleyjs_core_1.Logger.logInfo(loggerCategory, `Querying iModel by name ${iModelName}`, () => ({ contextId }));
            const imodels = await this.get(requestContext, contextId, new IModelQuery().byName(iModelName));
            requestContext.enter();
            bentleyjs_core_1.Logger.logTrace(loggerCategory, `Queried iModel by name ${iModelName}`, () => ({ contextId }));
            if (imodels.length > 0) {
                imodel = imodels[0];
            }
            else {
                bentleyjs_core_1.Logger.logTrace(loggerCategory, `iModel by name: iModel ${iModelName} not found`, () => ({ contextId }));
                throw new Error(`iModel by name: iModel ${iModelName} not found`);
            }
        }
        return imodel;
    }
    /** Get the [[InitializationState]] for the specified iModel. See [iModel creation]($docs/learning/iModelHub/iModels/CreateiModel.md).
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @returns State of the seed file initialization.
     * @throws [[IModelHubError]] with [IModelHubStatus.FileDoesNotExist]($bentley) if the seed file was not found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async getInitializationState(requestContext, iModelId) {
        const seedFiles = await this._seedFileHandler.get(requestContext, iModelId, new SeedFileQuery().latest());
        requestContext.enter();
        if (seedFiles.length < 1)
            throw new Errors_1.IModelHubError(bentleyjs_core_1.IModelHubStatus.FileDoesNotExist);
        return seedFiles[0].initializationState;
    }
    /**
     * Augment update options with defaults returned by the DefaultIModelCreateOptionsProvider. The options passed in by clients override any defaults where necessary.
     * @param options Options the caller wants to augment with the defaults.
     * @returns Promise resolves after the defaults are setup.
     */
    async setupOptionDefaults(options) {
        if (!IModelsHandler._defaultCreateOptionsProvider)
            IModelsHandler._defaultCreateOptionsProvider = new DefaultIModelCreateOptionsProvider();
        return IModelsHandler._defaultCreateOptionsProvider.assignOptions(options);
    }
    /** Wait until the iModel is initialized.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param imodel iModel instance that will be returned if initialization is successful.
     * @param timeOutInMilliseconds Maximum time to wait for the initialization.
     */
    async waitForInitialization(requestContext, contextId, imodel, timeOutInMilliseconds) {
        requestContext.enter();
        const errorMessage = "iModel initialization failed";
        const retryDelay = timeOutInMilliseconds / 10;
        for (let retries = 10; retries > 0; --retries) {
            try {
                const initState = await this.getInitializationState(requestContext, imodel.id);
                requestContext.enter();
                if (initState === InitializationState.Successful) {
                    bentleyjs_core_1.Logger.logTrace(loggerCategory, "Created iModel", () => ({ contextId, iModelId: imodel.id }));
                    imodel.initialized = true;
                    return imodel;
                }
                if (initState !== InitializationState.NotStarted && initState !== InitializationState.Scheduled) {
                    bentleyjs_core_1.Logger.logWarning(loggerCategory, errorMessage);
                    throw new Errors_1.IModelHubError(bentleyjs_core_1.IModelHubStatus.SeedFileInitializationFailed, `Seed file initialization failed with status ${InitializationState[initState]}`);
                }
                await new Promise((resolve) => setTimeout(resolve, retryDelay));
                requestContext.enter();
            }
            catch (err) {
                requestContext.enter();
                bentleyjs_core_1.Logger.logWarning(loggerCategory, errorMessage);
                throw err;
            }
        }
        bentleyjs_core_1.Logger.logWarning(loggerCategory, errorMessage);
        throw Errors_1.IModelHubClientError.initializationTimeout();
    }
    /**
     * Verifies iModel extent
     * @param extent iModel extent
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError] if extent is invalid.
     */
    validateExtent(extent) {
        if (extent.length === 0)
            return;
        if (extent.length !== IModelsHandler._imodelExtentLength)
            throw Errors_1.IModelHubClientError.invalidArgument("extent");
        this.validateExtentCoordinate(extent[0], IModelsHandler._imodelExtentLatitudeLimit);
        this.validateExtentCoordinate(extent[1], IModelsHandler._imodelExtentLongitudeLimit);
        this.validateExtentCoordinate(extent[2], IModelsHandler._imodelExtentLatitudeLimit);
        this.validateExtentCoordinate(extent[3], IModelsHandler._imodelExtentLongitudeLimit);
    }
    /**
     * Validates iModel extent coordinate
     * @param coordinate iModel extent coordinate
     * @param limit Coordinate limit value (latitude/longitude max value)
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError] if coordinate is invalid.
     */
    validateExtentCoordinate(coordinate, limit) {
        if (coordinate < -limit || coordinate > limit)
            throw Errors_1.IModelHubClientError.invalidArgument("extent");
    }
    /** Create an iModel from given seed file. See [iModel creation]($docs/learning/iModelHub/iModels/CreateiModel.md).
     * This method does not work on browsers. If iModel creation fails before finishing file upload, partially created iModel is deleted. This method is not supported in iModelBank.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param name Name of the iModel on the Hub.
     * @param createOptions Optional arguments for iModel creation.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have CreateiModel permission.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async create(requestContext, contextId, name, createOptions) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Creating iModel", () => ({ contextId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("contextId", contextId);
        Errors_1.ArgumentCheck.defined("name", name);
        createOptions = createOptions || {};
        await this.setupOptionDefaults(createOptions);
        if (createOptions.extent)
            this.validateExtent(createOptions.extent);
        if (createOptions.path) {
            createOptions.template = undefined;
            if (typeof window !== "undefined")
                throw Errors_1.IModelHubClientError.browser();
            if (!this._fileHandler)
                throw Errors_1.IModelHubClientError.fileHandler();
            if (!this._fileHandler.exists(createOptions.path) || this._fileHandler.isDirectory(createOptions.path))
                throw Errors_1.IModelHubClientError.fileNotFound();
        }
        const template = IModelsHandler._defaultCreateOptionsProvider.templateToString(createOptions);
        const imodel = await this.createIModelInstance(requestContext, contextId, name, createOptions.description, template, createOptions.iModelType, createOptions.extent);
        requestContext.enter();
        if (createOptions.template === iModelTemplateEmpty) {
            return imodel;
        }
        if (!createOptions.template) {
            try {
                await this._seedFileHandler.uploadSeedFile(requestContext, imodel.id, createOptions.path, createOptions.description, createOptions.progressCallback);
            }
            catch (err) {
                await this.delete(requestContext, contextId, imodel.id);
                throw err;
            }
            requestContext.enter();
        }
        return this.waitForInitialization(requestContext, contextId, imodel, createOptions.timeOutInMilliseconds);
    }
    /** Update iModel's name and/or description
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param imodel iModel to update. See [[HubIModel]].
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have CreateiModel permission.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel does not exist.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelIsNotInitialized]$(bentley) if iModel is not initialized.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelAlreadyExists]$(bentley) if iModel with specified name already exists.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, contextId, imodel) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Updating iModel", () => ({ contextId, iModelId: imodel.id }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("contextId", contextId);
        const updatedIModel = await this._handler.postInstance(requestContext, HubIModel, this.getRelativeUrl(contextId, imodel.id), imodel);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Updated iModel", () => ({ contextId, iModelId: imodel.id }));
        return updatedIModel;
    }
    /** Method to download the seed file for iModel. This will download the original seed file, that was uploaded when creating iModel. To download a file that was updated with ChangeSets on iModelHub, see [[BriefcaseHandler.download]].
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param path Path to download the seed file to, including file name.
     * @param progressCallback Callback for tracking progress.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async download(requestContext, iModelId, path, progressCallback) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Started downloading seed file", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.defined("path", path);
        if (typeof window !== "undefined")
            throw Errors_1.IModelHubClientError.browser();
        if (!this._fileHandler)
            throw Errors_1.IModelHubClientError.fileHandler();
        const seedFiles = await this._seedFileHandler.get(requestContext, iModelId, new SeedFileQuery().selectDownloadUrl().latest());
        requestContext.enter();
        if (!seedFiles || !seedFiles[0] || !seedFiles[0].downloadUrl)
            throw Errors_1.IModelHubError.fromId(bentleyjs_core_1.IModelHubStatus.FileDoesNotExist, "Failed to get seed file.");
        await this._fileHandler.downloadFile(requestContext, seedFiles[0].downloadUrl, path, parseInt(seedFiles[0].fileSize, 10), progressCallback);
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Finished downloading seed file", () => ({ iModelId }));
    }
}
exports.IModelsHandler = IModelsHandler;
IModelsHandler._imodelExtentLength = 4;
IModelsHandler._imodelExtentLatitudeLimit = 90;
IModelsHandler._imodelExtentLongitudeLimit = 180;
/**
 * Handler for managing [[HubIModel]] instance. Use [[IModelHubClient.IModel]] to get an instance of this handler.
 * @note Use [[IModelHubClient.IModels]] if multiple iModels per context are supported.
 * @beta
 */
class IModelHandler {
    /**
     * Constructor for IModelHandler. Should use @see IModelClient instead of directly constructing this.
     * @param handler Handler for managing [[HubIModel]] instances.
     * @note Use [[IModelHubClient.IModels]] if multiple iModels per context are supported.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /**
     * Get iModel that belong to the specified context.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @returns [[HubIModel]] instances that match the query.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel does not exist.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, contextId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying iModel", () => ({ contextId }));
        const query = new IModelQuery().orderBy("CreatedDate+asc").top(1);
        const imodels = await this._handler.get(requestContext, contextId, query);
        if (imodels.length < 1)
            throw new Errors_1.IModelHubError(bentleyjs_core_1.IModelHubStatus.iModelDoesNotExist);
        return imodels[0];
    }
    /**
     * Delete an iModel from a context. This method is not supported in iModelBank.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel does not exist.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have DeleteiModel permission.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async delete(requestContext, contextId) {
        const imodel = await this.get(requestContext, contextId);
        await this._handler.delete(requestContext, contextId, imodel.id);
    }
    /**
     * Get the [[InitializationState]] for the specified iModel. See [iModel creation]($docs/learning/iModelHub/iModels/CreateiModel.md).
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @returns State of the seed file initialization.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel does not exist.
     * @throws [[IModelHubError]] with [IModelHubStatus.FileDoesNotExist]($bentley) if the seed file was not found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async getInitializationState(requestContext, contextId) {
        const imodel = await this.get(requestContext, contextId);
        return this._handler.getInitializationState(requestContext, imodel.id);
    }
    /**
     * Create an iModel from given seed file. In most cases [BriefcaseManager.create]($backend) should be used instead. See [iModel creation]($docs/learning/iModelHub/iModels/CreateiModel.md).
     *
     * This method does not work on browsers. If iModel creation fails before finishing file upload, partially created iModel is deleted. This method is not supported in iModelBank.
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param name Name of the iModel on the Hub.
     * @param createOptions Optional arguments for iModel creation.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have CreateiModel permission.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async create(requestContext, contextId, name, createOptions) {
        requestContext.enter();
        let imodelExists = true;
        try {
            await this.get(requestContext, contextId);
            requestContext.enter();
        }
        catch (err) {
            if (err instanceof Errors_1.IModelHubError && err.errorNumber === bentleyjs_core_1.IModelHubStatus.iModelDoesNotExist)
                imodelExists = false;
            else
                throw err;
        }
        if (imodelExists)
            throw new Errors_1.IModelHubError(bentleyjs_core_1.IModelHubStatus.iModelAlreadyExists);
        return this._handler.create(requestContext, contextId, name, createOptions);
    }
    /**
     * Update iModel's name and/or description
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param imodel iModel to update. See [[HubIModel]].
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have CreateiModel permission.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel does not exist.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelIsNotInitialized]$(bentley) if iModel is not initialized.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelAlreadyExists]$(bentley) if iModel with specified name already exists.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, contextId, imodel) {
        return this._handler.update(requestContext, contextId, imodel);
    }
    /**
     * Method to download the seed file for iModel. This will download the original seed file, that was uploaded when creating iModel. To download a file that was updated with ChangeSets on iModelHub, see [[BriefcaseHandler.download]].
     * @param requestContext The client request context.
     * @param contextId Id for the iModel's context. For iModelHub it should be the id of the iTwin context ([[Project]] or [[Asset]]).
     * @param path Path where seed file should be downloaded, including filename.
     * @param progressCallback Callback for tracking progress.
     * @throws [[IModelHubError]] with [IModelHubStatus.iModelDoesNotExist]$(bentley) if iModel does not exist.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async download(requestContext, contextId, path, progressCallback) {
        const imodel = await this.get(requestContext, contextId);
        await this._handler.download(requestContext, imodel.id, path, progressCallback);
    }
}
exports.IModelHandler = IModelHandler;
//# sourceMappingURL=iModels.js.map